#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#

import tempfile
import types
from nose import SkipTest

from .utils import needs_uge
from .utils import generate_random_string
from .utils import create_config_file
from .utils import load_values

from uge.api.qconf_api import QconfApi
from uge.config.config_manager import ConfigManager
from uge.log.log_manager import LogManager
from uge.exceptions.object_not_found import ObjectNotFound
from uge.exceptions.object_already_exists import ObjectAlreadyExists

create_config_file()
API = QconfApi()
HOST_GROUP_NAME = '@%s' % generate_random_string(6)
CONFIG_MANAGER = ConfigManager.get_instance()
HOST_NAME = CONFIG_MANAGER['host']
LOG_MANAGER = LogManager.get_instance()
VALUES_DICT = load_values('test_values.json')
print(VALUES_DICT)


@needs_uge
def test_generate_hgrp():
    hgrp = API.generate_hgrp(HOST_GROUP_NAME)
    assert (hgrp.data['group_name'] == HOST_GROUP_NAME)


def test_list_hgrps():
    hgrpl = API.list_hgrps()
    assert (hgrpl is not None)


def test_add_hgrp():
    hgrp = API.add_hgrp(name=HOST_GROUP_NAME, data={'hostlist': HOST_NAME})
    assert (hgrp.data['group_name'] == HOST_GROUP_NAME)


def test_object_already_exists():
    try:
        hgrpl = API.list_hgrps()
        if len(hgrpl):
            hgrp = API.add_hgrp(name=hgrpl[0])
            assert (False)
        else:
            raise SkipTest('There are no configured UGE host groups.')
    except ObjectAlreadyExists as ex:
        # ok
        pass


def test_object_not_found():
    try:
        hgrp = API.get_hgrp('__non_existent_host_group__')
        assert (False)
    except ObjectNotFound as ex:
        # ok
        pass


def test_get_hgrp():
    hgrpl = API.list_hgrps()
    if len(hgrpl):
        hgrp = API.get_hgrp(hgrpl[0])
        assert (hgrp.data['group_name'] == hgrpl[0])
    else:
        raise SkipTest('There are no configured UGE host groups.')


def test_generate_hgrp_from_json():
    hgrpl = API.list_hgrps()
    if len(hgrpl):
        hgrp = API.get_hgrp(hgrpl[0])
    else:
        raise SkipTest('There are no configured UGE host groups.')

    json = hgrp.to_json()
    hgrp2 = API.generate_object(json)
    assert (hgrp2.__class__.__name__ == hgrp.__class__.__name__)
    for key in list(hgrp.data.keys()):
        v = hgrp.data[key]
        v2 = hgrp2.data[key]
        if type(v) == list:
            assert (len(v) == len(v2))
            for s in v:
                assert (v2.count(s) == 1)


def test_modify_hgrp():
    hgrpl = API.list_hgrps()
    if len(hgrpl):
        hgrp = API.get_hgrp(hgrpl[0])
        original_host_list = hgrp.data['hostlist']
        hgrp2 = API.modify_hgrp(name=hgrpl[0], data={'hostlist': HOST_NAME})
        assert (hgrp2.data['hostlist'].count(HOST_NAME) == 1)
        hgrp3 = API.modify_hgrp(name=hgrpl[0], data={'hostlist': original_host_list})
        assert (hgrp3.data['hostlist'] == original_host_list)
    else:
        raise SkipTest('There are no configured UGE host groups.')


def test_get_hgrps():
    hgrpl = API.list_hgrps()
    hgrps = API.get_hgrps()
    for hgrp in hgrps:
        print("#############################################")
        print(hgrp.to_uge())
        assert (hgrp.data['group_name'] in hgrpl)


def test_write_hgrps():
    try:
        tdir = tempfile.mkdtemp()
        print("*************************** " + tdir)
        hgrp_names = VALUES_DICT['hgrp_names']
        hgrps = API.get_hgrps()
        for hgrp in hgrps:
            print("Before #############################################")
            print(hgrp.to_uge())

        new_hgrps = []
        for name in hgrp_names:
            nhgrp = API.generate_hgrp(name=name)
            new_hgrps.append(nhgrp)
        API.mk_hgrps_dir(tdir)
        API.write_hgrps(new_hgrps, tdir)
        API.add_hgrps_from_dir(tdir)
        API.modify_hgrps_from_dir(tdir)
        hgrps = API.get_hgrps()
        for hgrp in hgrps:
            print("After #############################################")
            print(hgrp.to_uge())

        hgrps = API.list_hgrps()
        for name in hgrp_names:
            assert (name in hgrps)
            print("host group found: " + name)

    finally:
        API.delete_hgrps_from_dir(tdir)
        API.rm_hgrps_dir(tdir)


def test_add_hgrps():
    try:
        new_hgrps = []
        hgrp_names = VALUES_DICT['hgrp_names']
        for name in hgrp_names:
            nhgrp = API.generate_hgrp(name=name)
            new_hgrps.append(nhgrp)

        # print all host groups currently in the cluster
        hgrps = API.get_hgrps()
        for hgrp in hgrps:
            print("Before #############################################")
            print(hgrp.to_uge())

        # add host groups
        API.add_hgrps(new_hgrps)
        API.modify_hgrps(new_hgrps)

        # print all host groups currently in the cluster
        hgrps = API.get_hgrps()
        for hgrp in hgrps:
            print("After #############################################")
            print(hgrp.to_uge())

        # check that host groups have been added
        hgrps = API.list_hgrps()
        for name in hgrp_names:
            assert (name in hgrps)
            print("host group found: " + name)

    finally:
        API.delete_hgrps(new_hgrps)


def test_delete_hgrp():
    hgrpl = API.list_hgrps()
    if hgrpl.count(HOST_GROUP_NAME):
        API.delete_hgrp(HOST_GROUP_NAME)
        hgrpl2 = API.list_hgrps()
        assert (hgrpl2.count(HOST_GROUP_NAME) == 0)
    else:
        raise SkipTest('There UGE host group %s has not been added.' % HOST_GROUP_NAME)



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
from .qconf_object import QconfObject


class ResourceQuotaSet(QconfObject):
    """ This class encapsulates UGE resource quota set object. """

    #: Object version.
    VERSION = '1.0'

    #: Object name key.
    NAME_KEY = 'name'

    #: Object keys that must be provided by set.
    USER_PROVIDED_KEYS = ['name']

    #: Default values for required data keys.
    REQUIRED_DATA_DEFAULTS = {
        'description': None,
        'enabled': False,
        'limit': ['to slots=0'],
    }

    BOOL_KEY_MAP = QconfObject.get_bool_key_map(REQUIRED_DATA_DEFAULTS)
    DEFAULT_LIST_DELIMITER = ','
    LIST_KEY_MAP = {
        'limit': ',',
    }

    def __init__(self, name=None, data=None, metadata=None, json_string=None):
        """ 
        Class constructor. 

        :param name: Resource quota set name. If provided, it will override set name from data or JSON string parameters ('name' key).
        :type name: str

        :param data: Resource quota set data. If provided, it will override corresponding data from set JSON string representation.
        :type data: dict

        :param metadata: Resource quota set metadata. If provided, it will override corresponding metadata from set JSON string representation.
        :type metadata: dict

        :param json_string: Resource quota set JSON string representation.
        :type json_string: str

        :raises: **InvalidArgument** - in case metadata is not a dictionary, JSON string is not valid, or it does not contain dictionary representing an ResourceQuotaList object.
        """
        QconfObject.__init__(self, name=name, data=data, metadata=metadata, json_string=json_string)

    def to_uge(self):
        """ 
        Converts object to string acceptable as input for UGE qconf command.

        :returns: Object's UGE-formatted string.
        """
        lines = '{\n'
        for key in ['name', 'description', 'enabled']:
            value = self.data.get(key)
            lines += '%s %s\n' % (key, self.py_to_uge(key, value))
        limits = self.data.get('limit')
        for limit in limits:
            lines += 'limit %s\n' % limit
        lines += '}\n'
        return lines

    def py_to_uge(self, key, value):
        for (uge_value, py_value) in list(self.UGE_PYTHON_OBJECT_MAP.items()):
            if value == py_value and type(value) == type(py_value):
                return uge_value
        return value

    def to_dict(self, input_string):
        lines = input_string.split('\n')
        object_data = {}
        for line in lines:
            if not line:
                continue
            if line.startswith('{'):
                continue
            if line.startswith('}'):
                continue
            key_value = line.split()
            key = key_value[0]
            value = line.replace(key, '').strip()
            if key == 'limit':
                limits = object_data.get(key, [])
                limits.append(value)
                object_data[key] = limits
            else:
                object_data[key] = self.uge_to_py(key, value)
        return object_data

    def uge_to_py(self, key, value):
        uppercase_value = value.upper()
        for (uge_value, py_value) in list(self.UGE_PYTHON_OBJECT_MAP.items()):
            if uge_value == uppercase_value:
                return py_value
        return value



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
import re
from uge.exceptions.object_not_found import ObjectNotFound
from uge.objects.ar_object_factory import AdvanceReservationObjectFactory
from .dict_based_object_manager import DictBasedObjectManager


class AdvanceReservationManager(DictBasedObjectManager):
    AR_ERROR_REGEX_LIST = [
        (re.compile('.*No cluster queue or queue instance matches.*'), ObjectNotFound),
        (re.compile('.*no cqueue list defined.*'), ObjectNotFound),
    ]

    # Failure incorrectly classified as successful outcome
    AR_FAILURE_REGEX_LIST = []

    GENERATE_OBJECT_FACTORY_METHOD = AdvanceReservationObjectFactory.generate_advance_reservation
    OBJECT_NAME_KEY = 'id'
    OBJECT_CLASS_NAME = 'AdvanceReservation'
    OBJECT_CLASS_UGE_NAME = 'ar'

    def __init__(self, qrstat_executor, qrsub_executor, qrdel_executor):
        DictBasedObjectManager.__init__(self, qrstat_executor)


#############################################################################
# Testing.
if __name__ == '__main__':
    pass



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
try:
    import UserList
except ImportError:
    import collections as UserList
from uge.exceptions.invalid_argument import InvalidArgument
from .qconf_object import QconfObject


class QconfNameList(QconfObject, UserList.UserList):
    """ 
    This class represents list of names returned by various qconf commands. It supports
    all standard python list interfaces.
    """

    def __init__(self, name=None, data=None, metadata=None, json_string=None):
        """
        Class constructor. 

        :param name: List name. 
        :type name: str

        :param data: Name list data. If provided, it will override corresponding data from object's JSON string representation.
        :type data: list

        :param metadata: Name list metadata. If provided, it will override corresponding metadata from object's JSON string representation.
        :type metadata: dict

        :param json_string: Name list JSON string representation.
        :type json_string: str

        :raises: **InvalidArgument** - in case metadata is not a dictionary, JSON string is not valid, or it does not contain dictionary representing a QconfNameList object.
        """
        UserList.UserList.__init__(self)
        QconfObject.__init__(self, name=name, data=data, metadata=metadata, json_string=json_string)

    def check_input_data(self, data):
        if type(data) != list:
            raise InvalidArgument('Provided data is not a list: %s.' % str(data))

    def convert_data_to_uge_keywords(self, data):
        pass


#############################################################################
# Testing.
if __name__ == '__main__':
    name_list = QconfNameList(data=['xyz', 'abc'])
    print(name_list.__class__.__name__)
    name_list.append('admin1')
    name_list.append('admin2')
    print(name_list)
    for n in name_list:
        print('NAME: ', n)



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
from .qconf_object import QconfObject


class ClusterQueue(QconfObject):
    """ This class encapsulates UGE cluster queue object. """

    #: Object version.
    VERSION = '2.0'

    #: Object name key.
    NAME_KEY = 'qname'

    #: Object keys that must be provided by user.
    USER_PROVIDED_KEYS = ['qname']

    #: Default values for required data keys.
    REQUIRED_DATA_DEFAULTS = {
        'hostlist': None,
        'seq_no': 0,
        'load_thresholds': 'np_load_avg=1.75',
        'suspend_thresholds': None,
        'nsuspend': 1,
        'suspend_interval': '00:05:00',
        'priority': 0,
        'min_cpu_interval': '00:05:00',
        'qtype': 'BATCH INTERACTIVE',
        'ckpt_list': None,
        'pe_list': 'make',
        'jc_list': ['NO_JC', 'ANY_JC'],
        'rerun': False,
        'rerun_limit': 0,
        'rerun_limit_action': None,
        'slots': 1,
        'tmpdir': '/tmp',
        'shell': '/bin/sh',
        'prolog': None,
        'epilog': None,
        'shell_start_mode': 'unix_behavior',
        'starter_method': None,
        'suspend_method': None,
        'resume_method': None,
        'terminate_method': None,
        'notify': '00:00:60',
        'owner_list': None,
        'user_lists': None,
        'xuser_lists': None,
        'subordinate_list': None,
        'complex_values': None,
        'projects': None,
        'xprojects': None,
        'calendar': None,
        'initial_state': 'default',
        's_rt': float('inf'),
        'h_rt': float('inf'),
        'd_rt': float('inf'),
        's_cpu': float('inf'),
        'h_cpu': float('inf'),
        's_fsize': float('inf'),
        'h_fsize': float('inf'),
        's_data': float('inf'),
        'h_data': float('inf'),
        's_stack': float('inf'),
        'h_stack': float('inf'),
        's_core': float('inf'),
        'h_core': float('inf'),
        's_rss': float('inf'),
        'h_rss': float('inf'),
        's_vmem': float('inf'),
        'h_vmem': float('inf')
    }

    INT_KEY_MAP = QconfObject.get_int_key_map(REQUIRED_DATA_DEFAULTS)
    FLOAT_KEY_MAP = QconfObject.get_float_key_map(REQUIRED_DATA_DEFAULTS)
    DEFAULT_LIST_DELIMITER = ','
    LIST_KEY_MAP = {
        'slots': ',',
        'load_thresholds': ',',
        'suspend_thresholds': ',',
        'ckpt_list': ',',
        'pe_list': ',',
        'jc_list': ',',
        'owner_list': ',',
        'user_lists': ',',
        'xuser_lists': ',',
        'subordinate_list': ',',
        'complex_values': ',',
        'projects': ',',
        'xprojects': ',',
    }

    def __init__(self, name=None, data=None, metadata=None, json_string=None):
        """ 
        Class constructor. 

        :param name: Queue name. If provided, it will override queue name from data or JSON string parameters ('qname' key).
        :type name: str

        :param data: Queue data. If provided, it will override corresponding data from queue JSON string representation.
        :type data: dict

        :param metadata: Queue metadata. If provided, it will override corresponding metadata from queue JSON string representation.
        :type metadata: dict

        :param json_string: Queue JSON string representation.
        :type json_string: str

        :raises: **InvalidArgument** - in case metadata is not a dictionary, JSON string is not valid, or it does not contain dictionary representing a ClusterQueue object.
        """

        QconfObject.__init__(self, name=name, data=data, metadata=metadata, json_string=json_string)



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#

from logging import StreamHandler

from uge.config import config_manager


class StreamLogHandler(StreamHandler):
    """
    Class that enables logging into a stream. It can be used for logging
    into standard output or error.

    Usage:
        sh = StreamLogHandler(sys.stdout,)
    """

    def __init__(self, *args):
        """ Initialize log handler. """
        StreamHandler.__init__(self, *args)
        cm = config_manager.ConfigManager.get_instance()
        self.user = cm.get_user()
        self.host = cm.get_host()

    def emit(self, record):
        """ Emit the log record. """
        record.__dict__['user'] = self.user
        record.__dict__['host'] = self.host
        return StreamHandler.emit(self, record)



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
import re
import os
from uge.utility.uge_subprocess import UgeSubprocess
from uge.log.log_manager import LogManager
from uge.exceptions.qconf_exception import QconfException
from uge.exceptions.command_failed import CommandFailed
from uge.exceptions.qmaster_unreachable import QmasterUnreachable
from uge.exceptions.authorization_error import AuthorizationError
from uge.exceptions.object_not_found import ObjectNotFound


class QconfExecutor(object):
    QCONF_ERROR_REGEX_LIST = [
        (re.compile('.*unable to send message to qmaster.*'), QmasterUnreachable),
        (re.compile('.*must be manager.*'), AuthorizationError),
        (re.compile('denied.*'), AuthorizationError),
        (re.compile('.*does not exist.*'), ObjectNotFound),
        (re.compile('.*no.*defined.*'), ObjectNotFound),
        (re.compile('.*'), QconfException)
    ]
    QCONF_SUCCESS_REGEX_LIST = []  # for successful outcome incorrectly classified as failure
    QCONF_FAILURE_REGEX_LIST = []  # for failure incorrectly classified as successful outcome

    def __init__(self, sge_root, sge_cell, sge_qmaster_port, sge_execd_port):
        self.logger = LogManager.get_instance().get_logger(self.__class__.__name__)
        self.env_dict = {
            'SGE_ROOT': sge_root,
            'SGE_CELL': sge_cell,
            'SGE_QMASTER_PORT': str(sge_qmaster_port),
            'SGE_EXECD_PORT': str(sge_execd_port),
            'SGE_SINGLE_LINE': '1',
        }
        self.uge_version = None
        self.__configure()

    def __configure(self):
        self.logger.trace('Retrieving UGE version')
        uge_version = self.get_uge_version()
        self.logger.debug('UGE version: %s' % uge_version)

    def get_uge_version(self):
        if not self.uge_version:
            p = self.execute_qconf('-help')
            lines = p.get_stdout().split('\n')
            if not len(lines):
                raise QconfException('Cannot determine UGE version from output: %s' % p.get_stdout())
            self.uge_version = lines[0].split()[-1].split("_")[0].replace('(','').replace(')','')
            # if 2022.0.1 is used instead
            # self.uge_version = lines[0].split()[1]
        return self.uge_version

    def execute_qconf(self, cmd, error_regex_list=[], error_details=None, combine_error_lines=False,
                      success_regex_list=[], failure_regex_list=[]):
        try:
            command = '. %s/%s/common/settings.sh; qconf %s' % (
            self.env_dict['SGE_ROOT'], self.env_dict['SGE_CELL'], cmd)
            p = UgeSubprocess(command, env=self.env_dict)
            p.run()

            # In some cases successful outcome is actually a failure
            error = p.get_stderr()
            if error:
                for (pattern, qconfExClass) in failure_regex_list + QconfExecutor.QCONF_FAILURE_REGEX_LIST:
                    if pattern.match(error):
                        raise qconfExClass(error, error_details=error_details)
            return p
        except CommandFailed as ex:
            error = str(ex)
            if combine_error_lines:
                error = error.replace('\n', '; ')
            for (pattern, result) in success_regex_list + QconfExecutor.QCONF_SUCCESS_REGEX_LIST:
                if pattern.match(error):
                    self.logger.debug(
                        'Ignoring command failed for success pattern, replacing stdout with result: "%s"' % result)
                    p.stdout_ = result
                    return p
            for (pattern, qconfExClass) in error_regex_list + QconfExecutor.QCONF_ERROR_REGEX_LIST:
                if pattern.match(error):
                    raise qconfExClass(error, error_details=error_details)
            raise

    def execute_qconf_with_object(self, cmd, qconf_object, error_regex_list=[]):
        try:
            # fd, tmp_file_path = tempfile.mkstemp(text=True)
            # tmp_file = os.fdopen(fd, 'w')
            tmp_file_path = None
            tmp_dir_path = None
            tmp_file, tmp_file_path, tmp_dir_path = qconf_object.get_tmp_file()
            tmp_file_content = qconf_object.to_uge()
            tmp_file.write(tmp_file_content)
            tmp_file.flush()
            tmp_file.close()
            full_cmd = '%s %s' % (cmd, tmp_file_path)
            error_details = 'Object configuration file content:\n%s' % tmp_file_content
            self.execute_qconf(full_cmd, error_regex_list=error_regex_list, error_details=error_details)
        finally:
            if tmp_file_path is not None:
                os.remove(tmp_file_path)
            if tmp_dir_path is not None:
                os.rmdir(tmp_dir_path)

    def execute_qconf_with_dir(self, cmd, dir, error_regex_list=[]):
        if not os.path.isdir(dir):
            raise QconfException('%s is not a directory' % dir)
        full_cmd = '%s %s' % (cmd, dir)
        self.execute_qconf(full_cmd, error_regex_list=error_regex_list)


#############################################################################
# Testing.
if __name__ == '__main__':
    from uge.exceptions.command_failed import CommandFailed

    sge_root = os.environ.get('SGE_ROOT')
    if not sge_root:
        raise ConfigurationError('SGE_ROOT is not defined.')
    sge_cell = os.environ.get('SGE_CELL', 'default')
    if not sge_cell:
        raise ConfigurationError('SGE_CELL is not defined.')
    sge_qmaster_port = os.environ.get('SGE_QMASTER_PORT')
    if not sge_qmaster_port:
        raise ConfigurationError('SGE_QMASTER_PORT is not defined.')
    sge_execd_port = os.environ.get('SGE_EXECD_PORT')
    if not sge_execd_port:
        raise ConfigurationError('SGE_EXECD_PORT is not defined.')

    print('Configuration: SGE_ROOT=%s, SGE_CELL=%s, SGE_QMASTER_PORT=%s, SGE_EXECD_PORT=%s' % (
    sge_root, sge_cell, sge_qmaster_port, sge_execd_port))
    executor = QconfExecutor(sge_root=sge_root, sge_cell=sge_cell,
                             sge_qmaster_port=sge_qmaster_port,
                             sge_execd_port=sge_qmaster_port)
    try:
        print('Version: ', executor.get_uge_version())

        p = executor.execute_qconf('-sq all.q')
        print(p.get_stdout())
        print(p.get_exit_status())
    except CommandFailed as ex:
        print('Exit Status: ', ex.get_command_exit_status())
        print('Std Error  : ', ex.get_command_stderr())
